# Generated by Django 3.2.6 on 2021-08-20 16:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0022_auto_20210814_1828'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContentKeyValueStore',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('code', models.CharField(max_length=20)),
                ('is_active', models.BooleanField(default=True)),
                ('is_locked', models.BooleanField(default=True)),
                ('form_definition', models.JSONField(blank=True, default=dict)),
                ('content', models.JSONField(blank=True, default=dict)),
            ],
        ),
    ]
