# Generated by Django 2.1.8 on 2019-11-13 07:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0011_auto_20191006_1122'),
    ]

    operations = [
        migrations.AddField(
            model_name='productvariant',
            name='string_5',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='productvariant',
            name='string_1',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='productvariant',
            name='string_2',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='productvariant',
            name='string_3',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='productvariant',
            name='string_4',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
