# Generated by Django 3.2.12 on 2024-03-15 00:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0028_alter_analytic_options'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='basketwishlist',
            options={'ordering': ['-last_update']},
        ),
    ]
