# Generated by Django 2.1.8 on 2020-12-10 03:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0017_auto_20200309_0546'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='product',
            name='inventory_flag',
        ),
        migrations.AddField(
            model_name='product',
            name='inventory_type',
            field=models.CharField(default='T', max_length=1),
        ),
        migrations.AlterField(
            model_name='productcategory',
            name='level',
            field=models.PositiveIntegerField(db_index=True, editable=False),
        ),
        migrations.AlterField(
            model_name='productcategory',
            name='lft',
            field=models.PositiveIntegerField(db_index=True, editable=False),
        ),
        migrations.AlterField(
            model_name='productcategory',
            name='rght',
            field=models.PositiveIntegerField(db_index=True, editable=False),
        ),
    ]
