# Generated by Django 3.2.6 on 2021-08-20 17:02

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0023_contentkeyvaluestore'),
    ]

    operations = [
        migrations.AddField(
            model_name='contentkeyvaluestore',
            name='created',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='contentkeyvaluestore',
            name='created_local',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='contentkeyvaluestore',
            name='last_update',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='contentkeyvaluestore',
            name='last_update_local',
            field=models.DateTimeField(auto_now=True),
        ),
    ]
