# Generated by Django 3.2.6 on 2021-08-14 11:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0020_contentpage_contentpageitem_keyvaluestore_synclog_syncqueue'),
    ]

    operations = [
        migrations.RenameField(
            model_name='attribute',
            old_name='created',
            new_name='created_local',
        ),
        migrations.RenameField(
            model_name='attribute',
            old_name='last_update',
            new_name='last_update_local',
        ),
        migrations.RenameField(
            model_name='attributeitem',
            old_name='created',
            new_name='created_local',
        ),
        migrations.RenameField(
            model_name='attributeitem',
            old_name='last_update',
            new_name='last_update_local',
        ),
        migrations.RenameField(
            model_name='contentpage',
            old_name='created',
            new_name='created_local',
        ),
        migrations.RenameField(
            model_name='contentpage',
            old_name='last_update',
            new_name='last_update_local',
        ),
        migrations.RenameField(
            model_name='contentpageitem',
            old_name='created',
            new_name='created_local',
        ),
        migrations.RenameField(
            model_name='contentpageitem',
            old_name='last_update',
            new_name='last_update_local',
        ),
        migrations.RenameField(
            model_name='country',
            old_name='created',
            new_name='created_local',
        ),
        migrations.RenameField(
            model_name='country',
            old_name='last_update',
            new_name='last_update_local',
        ),
        migrations.RenameField(
            model_name='countrystate',
            old_name='created',
            new_name='created_local',
        ),
        migrations.RenameField(
            model_name='countrystate',
            old_name='last_update',
            new_name='last_update_local',
        ),
        migrations.RenameField(
            model_name='discountgroup',
            old_name='created',
            new_name='created_local',
        ),
        migrations.RenameField(
            model_name='discountgroup',
            old_name='last_update',
            new_name='last_update_local',
        ),
        migrations.RenameField(
            model_name='discountruleset',
            old_name='created',
            new_name='created_local',
        ),
        migrations.RenameField(
            model_name='discountruleset',
            old_name='last_update',
            new_name='last_update_local',
        ),
        migrations.RenameField(
            model_name='member',
            old_name='created',
            new_name='created_local',
        ),
        migrations.RenameField(
            model_name='member',
            old_name='last_update',
            new_name='last_update_local',
        ),
        migrations.RenameField(
            model_name='membergroup',
            old_name='created',
            new_name='created_local',
        ),
        migrations.RenameField(
            model_name='membergroup',
            old_name='last_update',
            new_name='last_update_local',
        ),
        migrations.RenameField(
            model_name='product',
            old_name='created',
            new_name='created_local',
        ),
        migrations.RenameField(
            model_name='product',
            old_name='last_update',
            new_name='last_update_local',
        ),
        migrations.RenameField(
            model_name='productcategory',
            old_name='created',
            new_name='created_local',
        ),
        migrations.RenameField(
            model_name='productcategory',
            old_name='last_update',
            new_name='last_update_local',
        ),
        migrations.RenameField(
            model_name='productvariant',
            old_name='created',
            new_name='created_local',
        ),
        migrations.RenameField(
            model_name='productvariant',
            old_name='last_update',
            new_name='last_update_local',
        ),
        migrations.RenameField(
            model_name='productvariantimage',
            old_name='created',
            new_name='created_local',
        ),
        migrations.RenameField(
            model_name='productvariantimage',
            old_name='last_update',
            new_name='last_update_local',
        ),
        migrations.RenameField(
            model_name='translation',
            old_name='created',
            new_name='created_local',
        ),
        migrations.RenameField(
            model_name='translation',
            old_name='last_update',
            new_name='last_update_local',
        ),
    ]
