# Generated by Django 3.2.6 on 2021-08-23 06:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0024_auto_20210821_0002'),
    ]

    operations = [
        migrations.AddField(
            model_name='product',
            name='local_image_version',
            field=models.CharField(blank=True, max_length=70, null=True),
        ),
        migrations.AddField(
            model_name='productcategory',
            name='local_image_version',
            field=models.CharField(blank=True, max_length=70, null=True),
        ),
        migrations.AddField(
            model_name='productvariant',
            name='local_image_version',
            field=models.CharField(blank=True, max_length=70, null=True),
        ),
    ]
