# Generated by Django 3.0.4 on 2020-03-09 05:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0016_auto_20191216_1359'),
    ]

    operations = [
        migrations.AlterField(
            model_name='productcategory',
            name='level',
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name='productcategory',
            name='lft',
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name='productcategory',
            name='rght',
            field=models.PositiveIntegerField(editable=False),
        ),
    ]
