# Generated by Django 3.2.12 on 2024-03-14 06:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0027_auto_20220516_0933'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='analytic',
            options={'ordering': ['-last_seen']},
        ),
    ]
