# Generated by Django 3.2.12 on 2022-05-16 02:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0026_auto_20211231_1342'),
    ]

    operations = [
        migrations.AddField(
            model_name='attributeitem',
            name='extras',
            field=models.JSONField(blank=True, db_index=True, default=dict),
        ),
        migrations.AlterField(
            model_name='attributeitem',
            name='slug',
            field=models.CharField(blank=True, db_index=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='attributeitem',
            name='value',
            field=models.CharField(max_length=255),
        ),
    ]
